// Generated by CoffeeScript 2.7.0
var pad;

pad = require('pad');

/*

Help Plugin
-----------

Display help when the user types "help" or runs commands without arguments. 
Command help is only displayed if a description was provided during the 
command registration. Additionnaly, a new `shell.help()` function is made available. 

Options passed during creation are:

-   `shell`        , (required) A reference to your shell application.
-   `introduction` , Print message 'Type "help" or press enter for a list of commands' if boolean `true`, or a custom message if a `string`

Usage

  app = shell()
  app.configure ->
    app.use shell.router shell: app
    app.use shell.help
      shell: app
      introduction: true

*/
module.exports = function(settings) {
  var shell, text;
  if (!settings.shell) {
    // Validation
    throw new Error('No shell provided');
  }
  shell = settings.shell;
  // Register function
  shell.help = function(req, res, next) {
    var i, len, route, routes, text;
    res.cyan('Available commands:');
    res.ln();
    routes = shell.routes;
    for (i = 0, len = routes.length; i < len; i++) {
      route = routes[i];
      text = pad(route.command, 20);
      if (route.description) {
        res.cyan(text).white(route.description).ln();
      }
    }
    return res.prompt();
  };
  // Register commands
  shell.cmd('help', 'Show this message', shell.help.bind(shell));
  shell.cmd('', shell.help.bind(shell));
  // Print introduction message
  if (shell.isShell && settings.introduction) {
    text = typeof settings.introduction === 'string' ? settings.introduction : 'Type "help" or press enter for a list of commands';
    return shell.styles.println(text);
  }
};
