// Generated by CoffeeScript 2.7.0
var Interface, crypto, fs, hash;

fs = require('fs');

crypto = require('crypto');

Interface = require('readline').Interface;

hash = function(value) {
  return crypto.createHash('md5').update(value).digest('hex');
};

/*

History plugin
==============

Persistent command history over multiple sessions. Options passed during creation are:

-   `shell`  , (required) A reference to your shell application.
-   `name`   , Identify your project history file, default to the hash of the exectuted file
-   `dir`    , Location of the history files, defaults to `"#{process.env['HOME']}/.node_shell"`

*/
module.exports = function(settings) {
  var e, file, json, shell, stream;
  if (!settings.shell) {
    // Validation
    throw new Error('No shell provided');
  }
  shell = settings.shell;
  // Only in shell mode
  if (!settings.shell.isShell) {
    return;
  }
  // Persist readline history
  if (settings.dir == null) {
    settings.dir = `${process.env['HOME']}/.node_shell`;
  }
  if (settings.name == null) {
    settings.name = hash(process.argv[1]);
  }
  file = `${settings.dir}/${settings.file}`;
  if (!fs.existsSync(settings.dir)) {
    // Create store directory
    fs.mkdirSync(settings.dir, 0o0700);
  }
  // Look for previous history
  if (fs.existsSync(file)) {
    try {
      json = fs.readFileSync(file, 'utf8') || '[]';
      settings.shell.interface().history = JSON.parse(json);
    } catch (error) {
      e = error;
      settings.shell.styles.red('Corrupted history file').ln();
    }
  }
  // Write new history
  stream = fs.createWriteStream(file, {
    flag: 'w'
  });
  Interface.prototype._addHistory = (function(parent) {
    return function() {
      var buffer;
      if (this.history.length) {
        buffer = Buffer.from(JSON.stringify(this.history), 'utf8');
        fs.writeSync(stream.fd, buffer, 0, buffer.length, 0);
      }
      return parent.apply(this, arguments);
    };
  })(Interface.prototype._addHistory);
  return null;
};
