const btnOpen = document.getElementById("btnOpen");
const btnGo = document.getElementById("btnGo");
const backToLargeWindow = document.getElementById("backToLargeWindow");
const useSmallWindowBtn = document.getElementById("useSmallBtn");
const stopBtnSmall = document.getElementById("stopImgSmall");
const playBtnSmall = document.getElementById("playImgSmall");
const moveBack = document.getElementById("moveBack");
const moveBackSmall = document.getElementById("moveBackSmall");
const moveForward = document.getElementById("moveForward");
const moveForwardSmall = document.getElementById("moveForwardSmall");
const filePathElement = document.getElementById("filePath");
const counter = document.getElementById("counter");
const counterSmall = document.getElementById("counterSmall");
const connectBtn = document.getElementById("connectBtn");
const disconnectBtn = document.getElementById("disconnectBtn");
const connectionStatus = document.getElementById("connectionStatus");
const connectionStatusSmall = document.getElementById("connectionStatusSmall");
const connectionToken = document.getElementById("connectionToken");
const controlBlock = document.getElementById("controlBlock");
const recentFilesElement = document.getElementById("recentFiles");

document.getElementById("infoIcon").addEventListener(
  "click",
  () => {
    toggleInfoBlock(1);
  },
  false
);
document.getElementById("closeInfo").addEventListener(
  "click",
  () => {
    toggleInfoBlock(0);
  },
  false
);
document.getElementById("openAppPage").addEventListener("click", openURL);
document.getElementById("openVLCPage").addEventListener("click", openURL);
document.getElementById("controllerPage").addEventListener("click", openURL);

function openURL(event) {
  window.electronAPI.openURL(event.target.getAttribute("data-url"));
}

const STATUS = {
  PLAYING: 1,
  STOPPED: 2,
};

let playbackStatus = null;
let connectionEstablished = false;

let recentFiles = [];

connectBtn.addEventListener(
  "click",
  async () => {
    const token = connectionToken.value.trim();
    if (!token) {
      connectionToken.className = "connectionTokenInputError";
      connectionStatus.textContent = "please enter valid password";
      document.getElementById("tokenHelpBlock").style.display = "block";
    } else {
      connectionToken.className = "connectionTokenInput";
      connectionStatus.textContent = "establishing connection .... ";
      document.getElementById("tokenHelpBlock").style.display = "none";
      window.electronAPI.useToken(token);
    }
  },
  false
);

disconnectBtn.addEventListener(
  "click",
  async () => {
    window.electronAPI.disconnect();
  },
  false
);

connectionToken.addEventListener(
  "focus",
  () => {
    connectionToken.className = "connectionTokenInput";
  },
  false
);

recentFilesElement.addEventListener("change", (e) => {
  const select = document.getElementById("recentFiles");
  const ind = select.value * 1;

  if (recentFiles[ind]) {
    setCurrentFile(recentFiles[ind].filePath);
    counter.textContent = formatDurationLong(recentFiles[ind].position);
  }
});

btnOpen.addEventListener(
  "click",
  async () => {
    const filePath = await window.electronAPI.openFile();

    if (filePath.toLowerCase().endsWith(".srt")) {
      alert("Please select a video file, not a subtitles file");
    } else {
      setCurrentFile(filePath);
    }
  },
  false
);

backToLargeWindow.addEventListener("click", () => {
  useSmallWindow(false);
});
useSmallWindowBtn.addEventListener("click", () => {
  useSmallWindow(true);
});

btnGo.addEventListener("click", goHandler);
stopBtnSmall.addEventListener("click", goHandler);
playBtnSmall.addEventListener("click", goHandler);

moveBack.addEventListener("click", moveBackHandler, false);
moveBackSmall.addEventListener("click", moveBackHandler, false);

moveForward.addEventListener("click", moveForwardHandler, false);
moveForwardSmall.addEventListener("click", moveForwardHandler, false);

window.electronAPI.handleHistory((event, history) => {
  recentFiles = history;
  if (history.length > 0) {
    for (let i = 0; i < history.length; i++) {
      const option = document.createElement("option");
      option.value = i + "";
      option.text = history[i].filePath;
      recentFilesElement.appendChild(option);
    }

    filePathElement.textContent = history[0].filePath;
    counter.textContent = formatDurationLong(history[0].position);
    controlBlock.classList.remove("controlBlockDisabled");
  } else {
    document.getElementById("recentFilesEmpty").textContent = "no recent files";
  }
});

window.electronAPI.handleStatusUpdate((event, value) => {
  const status = JSON.parse(value);
  if (status.time) {
    counter.textContent = formatDurationLong(status.time);
    counterSmall.textContent = formatDurationLong(status.time);
  }

  if (status.state === "playing") {
    setButtonState(STATUS.PLAYING);
  } else {
    setButtonState(STATUS.STOPPED);
  }
});

window.electronAPI.reportConnection((event, obj) => {
  connectionStatus.style.color = obj.color;
  connectionStatus.textContent = obj.text;
  connectionStatusSmall.style.color = obj.color;
  connectionStatusSmall.textContent = obj.text;

  if (obj.color !== "red") {
    connectionEstablished = true;
    document.getElementById("tokenHelpBlock").style.display = "none";

    document.getElementById("connectBtn").classList.add("hiddenBtn");
    document.getElementById("disconnectBtn").classList.remove("hiddenBtn");
  } else {
    connectionEstablished = false;

    if (obj.code === "showHelp") {
      document.getElementById("tokenHelpBlock").style.display = "block";
    }

    document.getElementById("connectBtn").classList.remove("hiddenBtn");
    document.getElementById("disconnectBtn").classList.add("hiddenBtn");
  }
});

window.electronAPI.reportDisconnect((event, value) => {
  connectionStatus.textContent = "disconnected";
  connectionStatus.style.color = "inherit";
  connectionEstablished = false;

  connectionStatusSmall.textContent = "disconnected";
  connectionStatusSmall.style.color = "inherit";

  document.getElementById("connectBtn").classList.remove("hiddenBtn");
  document.getElementById("disconnectBtn").classList.add("hiddenBtn");
});

window.electronAPI.reportSavedToken((_, value) => {
  connectionToken.value = value;
  if (value.trim() === "") {
    document.getElementById("tokenHelpBlock").style.display = "block";
  }
});

window.electronAPI.handleBadFile((_, value) => {
  filePathElement.classList.remove("validMediaFile");
  filePathElement.classList.add("badMediaFile");

  playbackStatus = null;
  setButtonState(STATUS.STOPPED);
  alert("File does not exit: " + value);
});

window.electronAPI.vlcCheck((_, value) => {
  if (!value) {
    alert(
      "VLC Player not found on your computer.\n\nPlease install VLC Player and then retry using this application."
    );
    window.electronAPI.quit();
  }
});

function formatDuration(val) {
  let sec_num = parseInt(val, 10);
  let hours = Math.floor(sec_num / 3600);
  let minutes = Math.floor((sec_num - hours * 3600) / 60);
  let seconds = sec_num - hours * 3600 - minutes * 60;
  return [hours, minutes, seconds];
}

function formatDurationLong(val) {
  const tmp = formatDuration(val);
  let hours = tmp[0];
  let minutes = tmp[1];
  let seconds = tmp[2];

  if (hours < 10) {
    hours = "0" + hours;
  }
  if (minutes < 10) {
    minutes = "0" + minutes;
  }
  if (seconds < 10) {
    seconds = "0" + seconds;
  }

  return hours + ":" + minutes + ":" + seconds;
}

async function goHandler() {
  if (filePathElement.innerText.trim() === "") {
    alert("Please select a file to play");
  } else {
    if (playbackStatus === null) {
      let go = connectionEstablished;
      if (!connectionEstablished) {
        if (
          confirm(
            "Connection to server was not yet established so subtitles position will be not be updated at subtitles-robot.com website.\n\nDo you want to start playback anyway?"
          )
        ) {
          go = true;
        }
      }

      if (go) {
        playbackStatus = "playing";
        window.electronAPI.go();

        document.getElementById("recentFilesContainer").style.display = "none";

        useSmallWindow(true);
        document.getElementById("smallWindowBlock").style.display = "block";

        setButtonState(STATUS.PLAYING);
      }
    } else {
      if (playbackStatus === "playing") {
        playbackStatus = "paused";
        window.electronAPI.pause();
        setButtonState(STATUS.STOPPED);
      } else {
        playbackStatus = "playing";
        window.electronAPI.play();
        setButtonState(STATUS.PLAYING);
      }
    }
  }
}

function setButtonState(isPlaying) {
  if (isPlaying === STATUS.PLAYING) {
    document.getElementById("playImg").classList.add("imgHidden");
    document.getElementById("stopImg").classList.remove("imgHidden");

    document.getElementById("playImgSmall").classList.add("imgHidden");
    document.getElementById("stopImgSmall").classList.remove("imgHidden");
  } else {
    document.getElementById("playImg").classList.remove("imgHidden");
    document.getElementById("stopImg").classList.add("imgHidden");

    document.getElementById("playImgSmall").classList.remove("imgHidden");
    document.getElementById("stopImgSmall").classList.add("imgHidden");
  }
}

function useSmallWindow(flag) {
  if (flag) {
    window.electronAPI.setWindowSize({ width: 900, height: 110 });

    document.getElementById("smallViewContainer").style.display = "block";
    document.getElementById("largeViewContainer").style.display = "none";
  } else {
    window.electronAPI.setWindowSize({ width: 900, height: 740 });

    document.getElementById("smallViewContainer").style.display = "none";
    document.getElementById("largeViewContainer").style.display = "block";
  }
}

async function moveBackHandler() {
  if (playbackStatus) {
    window.electronAPI.movePosition(-15);
  }
}

async function moveForwardHandler() {
  if (playbackStatus) {
    window.electronAPI.movePosition(+15);
  }
}

function setCurrentFile(filePath) {
  filePathElement.innerText = filePath;
  filePathElement.classList.add("validMediaFile");
  filePathElement.classList.remove("badMediaFile");

  window.electronAPI.setCurrentFile(filePath);

  if (filePath) {
    controlBlock.classList.remove("controlBlockDisabled");
  } else {
    controlBlock.classList.add("controlBlockDisabled");
  }
}

function toggleInfoBlock(mode) {
  if (mode === 1) {
    document.getElementById("infoBlock").style.display = "block";
    document.getElementById("centralBlockContainer").style.display = "none";
  } else {
    document.getElementById("infoBlock").style.display = "none";
    document.getElementById("centralBlockContainer").style.display = "block";
  }
}
